{ (C) 2001 Marek Wodzinski
 This code is licensed under GNU GPL
 File version: 1.0
 Last modification: Mon Oct 15 17:06:58 2001
   
 Changes:
 1.0 - initial release
}
      

uses rex;

type
  taddins=record
  	    nr:longint;
	    page:word;
	    data:array[0..8191] of byte;
	  end;
  
var
  addins,pom:twdbrecord;
  addintypes:twdbtype;
  dbpage:word;
  buf:tbufor;
  alladdins:array[0..7] of taddins;
  i:byte;
  inputfile:string;
  inputtype:byte;
  yestoall:boolean;
  addinnumber,k:integer;
  pagenumber:word;
  f:file;
  opened:byte;
  c:char;
  ofsecik,ile:word;


procedure print_help(msg:string);
begin
  if opened>0 then close_rex;
  if msg<>'' then writeln('** ',msg,' **');
  writeln('Rexaddin version 1.0 (C) 2001 Marek Wodzinski');
  writeln('Usage: rexaddin device [-a nr] [-r file.rex]|[-b raw.bin]|[-z raw.z80] [-y]');
  writeln('device      - /dev/ttyS0 or something like this:-)');
  writeln('-a nr       - addin number');
  writeln('-r file.rex - read image from .rex compatible addin file');
  writeln('-b raw.bin  - read image from simple raw binary file');
  writeln('-z raw.z80  - read image produced by z80-asm');
  writeln('-y          - if -a and (-r or -b) parameters are specified then skip');
  writeln('              confirmation question');
  halt;
end;  

procedure print_error(msg:string);
begin
  if opened>0 then close_rex;
  if msg<>'' then writeln('** ',msg,' **');
  halt(11);
end;  

procedure print_info(pp:twdbrecord);
var
  nr:cardinal;
  page:word;
  s1,s2:string;
  len:word;
begin
  if pp=nil then exit;
  nr:=pp^.data^[0]+256*pp^.data^[1]+65536*(pp^.data^[2]+256*pp^.data^[3]);
  page:=pp^.data^[4]+256*pp^.data^[5];
  len:=pp^.data^[10]+256*pp^.data^[11];
  s1:=buftostr(@pp^.data^[12],len);
  s2:=buftostr(@pp^.data^[14+len],pp^.data^[12+len]+256*pp^.data^[13+len]);
  if (addinnumber=-1) or (addinnumber=nr) then
  begin
    writeln('Addin nr: ',nr,' located at page: ',page);
    writeln('    name: ',s1);
    writeln('   extra: ',s2);
    writeln('-----------------------------------------------------------');
  end;
  alladdins[i].nr:=nr;
  alladdins[i].page:=page;
  inc(i);
end;

begin
  opened:=0;
  if paramcount<1 then print_help('');
  if not open_rex(paramstr(1)) then halt;
  opened:=1;
  
  //initialization
  inputfile:='';
  inputtype:=0;
  yestoall:=false;
  addinnumber:=-1;  
  for i:=0 to 7 do
    alladdins[i].nr:=-1;
  
  //reading parameters;
  for i:=2 to paramcount do
  begin
    if paramstr(i)='-a' then
    begin
      if paramcount=i then print_help('Wrong parametr');
      val(paramstr(i+1),addinnumber,k);
    end else
    if paramstr(i)='-r' then
    begin
      if paramcount=i then print_help('Wrong parametr');
      inputtype:=1;
      inputfile:=paramstr(i+1);
    end else
    if paramstr(i)='-b' then
    begin
      if paramcount=i then print_help('Wrong parametr');
      inputtype:=0;
      inputfile:=paramstr(i+1);
    end else
    if paramstr(i)='-z' then
    begin
      if paramcount=i then print_help('Wrong parametr');
      inputtype:=2;
      inputfile:=paramstr(i+1);
    end else
    if paramstr(i)='-y' then
    begin
      if (addinnumber>0) and (inputfile<>'') then yestoall:=true
      else print_help('Wrong parametr'); 
    end;
  end;

  //read databases from rex
  find_db(150,addintypes,dbpage);
  read_db(dbpage,addins);
  
  //write info about addins and write it to array
  writeln('-- ADDIN UPDATER ------------------------------------------');
  i:=0;
  pom:=addins;
  while pom<>nil do
  begin  
    print_info(pom);
    pom:=pom^.next;
  end;

  //database no longer needed
  destroy_db_records(addins);
  destroy_db_type(addintypes);
  
  //update???
  if not yestoall then
  begin
    write('Do you want to update some addin (y/N)?');
    read(c);
    writeln;
    if upcase(c)<>'Y' then
    begin
      close_rex;
      halt(0);
    end;
  end;
  
  //addin number
  if addinnumber=-1 then
  begin
    write('Please enter addin number :');
    readln(addinnumber);
  end;
  
  //check for correct number
  pagenumber:=0;
  for i:=0 to 7 do
    if alladdins[i].nr=addinnumber then pagenumber:=alladdins[i].page;

  if pagenumber=0 then print_help('Unknown page or bad addin number');  

  //file name and type
  if inputfile='' then
  begin
    write('Please enter addin file name :');
    readln(inputfile);
    write('This is raw (b)inary, (r)ex or (z)80-asm format (R/b/z)?');
    read(c);
    writeln;
    if upcase(c)='R' then inputtype:=1;
    if upcase(c)='Z' then inputtype:=2;
  end;
  
  //read data from file
  assign(f,inputfile);
  reset(f,1);
  //if rex format then skip headers/icon
  if inputtype=1 then
  begin
    //skip addin name etc.
    repeat
      blockread(f,c,1,k);
    until c=#0;
    //skip ADD and icon
    blockread(f,buf,163,k);
  end;
  //if z80-asm format then skip header
  if inputtype=2 then
    blockread(f,buf,10,k);

  //read from file directly into array
  for i:=0 to 7 do
    if alladdins[i].nr=addinnumber then
    begin
      fillbyte(alladdins[i].data,8192,255);
      blockread(f,alladdins[i].data,8192,k);
    end;
  close(f);
  
  //now we must read all addins before erasing flash
  for i:=0 to 7 do
    if (alladdins[i].nr<>addinnumber) and (alladdins[i].nr<>-1) then
    begin
      if dsfr(@buf,alladdins[i].page,8192,0)<>8192 then print_error('error reading flash!');
      move(buf[0],alladdins[i].data[0],8192);
      writeln('addin ',alladdins[i].nr,' read succesfully');
    end;

  //erase addin flash
  write('Erasing flash...');
  dsfe($80);
  writeln(' done');
  
  //write addins to flash
  for i:=0 to 7 do
    if alladdins[i].nr<>-1 then
    begin
      write('writing addin ',alladdins[i].nr,' to page ',alladdins[i].page);
      ofsecik:=0;
      while ofsecik<8192 do
      begin
        ile:=8192-ofsecik;
	if ile>509 then ile:=509;
	dsfw(@alladdins[i].data[ofsecik],alladdins[i].page,ile,ofsecik);
	ofsecik:=ofsecik+ile;
	write('.');
      end;
      writeln;
    end;
  
  close_rex;
end.
							